import typer
from pathlib import Path
from geopic_tag_reader import reader
from geopic_tag_reader.model import PictureType
from typing import Optional
import pyexiv2  # type: ignore
from geopic_tag_reader.i18n import init as i18n_init

app = typer.Typer(help="GeoPicTagReader")


@app.command()
def read(
    image: Path = typer.Option(..., help="Path to your JPEG image file"),
    ignore_exiv2_errors: bool = typer.Option(False, "--ignore-exiv2-errors", help="Do not stop execution even if Exiv2 throws errors"),
    lang: str = typer.Option("en", help="Lang code (2 letters) to use for printing messages"),
):
    """Reads EXIF metadata from a picture file, and prints results"""

    with open(image, "rb") as img:
        pyexiv2.set_log_level(4 if ignore_exiv2_errors else 2)

        metadata = reader.readPictureMetadata(img.read(), lang)

        _ = i18n_init(lang)
        print(_("Latitude:"), metadata.lat)
        print(_("Longitude:"), metadata.lon)
        print(_("Timestamp:"), metadata.ts.isoformat())
        print(_("Heading:"), metadata.heading)
        print(_("Type:"), metadata.type)
        print(_("Make:"), metadata.make)
        print(_("Model:"), metadata.model)
        print(_("Focal length:"), metadata.focal_length)
        print(_("Crop parameters:"), metadata.crop)
        print(_("Pitch:"), metadata.pitch)
        print(_("Roll:"), metadata.roll)

        if len(metadata.tagreader_warnings) > 0:
            print(_("Warnings raised by reader:"))
            for w in metadata.tagreader_warnings:
                print(" - " + w)


@app.command()
def write(
    input: Path = typer.Option(help="Path to your JPEG image file"),
    output: Optional[Path] = typer.Option(
        default=None, help="Output path where to write the updated image file. If not present, the input file will be overriten."
    ),
    capture_time: Optional[str] = typer.Option(
        default=None,
        help="override capture time of the image, formated in isoformat, like '2023-06-01T12:48:01Z'. Note that if no timezone offset is defined, the datetime will be taken as local time and localized using the picture position if available.",
    ),
    longitude: Optional[float] = typer.Option(
        default=None,
        help="override longitude of the image, in decimal degrees (WGS84 / EPSG:4326) (like `2.3522219` for Paris)",
    ),
    latitude: Optional[float] = typer.Option(
        default=None,
        help="override latitude of the image, in decimal degrees (WGS84 / EPSG:4326) (like `48.856614` for Paris)",
    ),
    picture_type: Optional[PictureType] = typer.Option(
        default=None,
        help="type of picture, `equirectangular` for 360° pictures, `flat` otherwise",
    ),
    lang: str = typer.Option("en", help="Lang code (2 letters) to use for printing messages"),
):
    """Override certain exiftags of a picture and write a new picture in another file"""
    from geopic_tag_reader import writer
    from dateutil.parser import parse

    capture_dt = parse(capture_time) if capture_time else None

    with open(input, "rb") as raw_input:
        updated_pic = writer.writePictureMetadata(
            raw_input.read(),
            writer.PictureMetadata(capture_time=capture_dt, longitude=longitude, latitude=latitude, picture_type=picture_type),
            lang,
        )

        out = output or input
        if output is None:
            print(f"Ovewriting {input} metadatas")
        with open(out, "wb") as o:
            o.write(updated_pic)


if __name__ == "__main__":
    app()



import gettext
import os
from typing import Callable

localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), "translations")


def init(lang_code: str = "en") -> Callable[[str], str]:
    lang = gettext.translation("geopic_tag_reader", localedir, languages=[lang_code], fallback=True)
    lang.install()
    return lang.gettext



from typing import Optional, Dict, List

# List of equirectangular cameras (make, model)
# https://en.wikipedia.org/wiki/List_of_omnidirectional_(360-degree)_cameras
EQUIRECTANGULAR_MODELS: Dict[str, List[str]] = {
    "Panono": ["Panono"],
    "Vuze": ["Vuze"],
    "Vuze+": ["Vuze+"],
    "BUBL": ["Bublcam"],
    "Ricoh": ["Theta", "Theta m15", "Theta S", "Theta SC", "Theta SC2", "Theta V", "Theta Z1"],
    "Insta360": ["4K", "Nano", "Air", "One", "Pro", "One X", "One R", "X3", "Titan"],
    "Samsung": ["Gear360"],
    "LG": ["360 CAM"],
    "MadV": ["Madventure 360"],
    "Nikon": ["Keymission 360"],
    "Xiaomi": ["米家全景相机"],
    "小蚁(YI)": ["小蚁VR全景相机"],
    "Giroptic iO": ["Giroptic iO"],
    "Garmin": ["Virb 360"],
    "Nokia": ["OZO"],
    "Z Cam": ["S1 Pro", "V1 Pro"],
    "Rylo": ["Rylo"],
    "GoPro": ["Fusion", "Max"],
    "FXG": ["SEIZE", "FM360 Duo"],
}


def is_360(make: Optional[str] = None, model: Optional[str] = None, width: Optional[str] = None, height: Optional[str] = None) -> bool:
    """
    Checks if given camera is equirectangular (360°) based on its make, model and dimensions (width, height).

    >>> is_360()
    False
    >>> is_360("GoPro")
    False
    >>> is_360("GoPro", "Max 360")
    True
    >>> is_360("GoPro", "Max 360", "2048", "1024")
    True
    >>> is_360("GoPro", "Max 360", "1024", "768")
    False
    >>> is_360("RICOH", "THETA S", "5376", "2688")
    True
    """

    # Check make and model are defined
    if not make or not model:
        return False

    # Check width and height are equirectangular
    if not ((width is None or height is None) or int(width) == 2 * int(height)):
        return False

    # Find make
    matchMake = next((m for m in EQUIRECTANGULAR_MODELS.keys() if make.lower() == m.lower()), None)
    if matchMake is None:
        return False

    # Find model
    return any(model.lower().startswith(m.lower()) for m in EQUIRECTANGULAR_MODELS[matchMake])
